# -*- coding: utf-8 -*-
"""
Created on Thu Aug 13 20:21:44 2015

@author: Sven
"""

import MonteCarloStandard as mcs
import MonteCarloNumpy as mcn
import MonteCarloCtypes as mcc

import numpy as np
from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import time


# Daten für Schaubild erstellen
x = np.arange(-0.04, 0.04, 0.001)
y = np.arange(-0.04, 0.04, 0.001)
X,Y = np.meshgrid(x, y)

Z = mcs.ackley(X, Y)

python_time = []
numpy_time = []
ctypes_time = []

linksunten = [-2.0,-2.0]
rechtsoben = [2.0,2.0]
n = 500000

# Später mit 200 Durchgängen
for i in range(200):
    start = time.time()
    px,py,pz = mcs.montecarlo(linksunten, rechtsoben, n, func=mcs.ackley)
    python_time.append(time.time()-start)
    
    start = time.time()
    npx,npy,npz = mcn.montecarlo(linksunten, rechtsoben, n, func=mcn.ackley)
    numpy_time.append(time.time()-start)
    
    start = time.time()
    cpx, cpy, cpz = mcc.montecarlo(linksunten, rechtsoben, n)
    ctypes_time.append(time.time()-start)

# Zeitdaten auswerten

n, bins, patches = plt.hist(python_time, 10, normed=False, facecolor='red', alpha=0.75)

plt.xlabel('Zeit in s')
plt.ylabel(u'Häufigkeit')
plt.title(r'Dauer Monte-Carlo-Suche nur mit Python')
plt.grid(True)
plt.show()

n, bins, patches = plt.hist(numpy_time, 10, normed=False, facecolor='blue', alpha=0.75)

plt.xlabel('Zeit in s')
plt.ylabel(u'Häufigkeit')
plt.title(r'Dauer Monte-Carlo-Suche mit Numpy')
plt.grid(True)
plt.show()

n, bins, patches = plt.hist(ctypes_time, 10, normed=False, facecolor='green', alpha=0.75)

plt.xlabel('Zeit in s')
plt.ylabel(u'Häufigkeit')
plt.title(r'Dauer Monte-Carlo-Suche mit Auslagerungen in C++')
plt.grid(True)
plt.show()



# Ergebnis ausgeben
print '--Auswertung------------------------------------------------------------'
print '----Schleife in Python--------------------------------------------------'
print 'Durchschnittliche Dauer: ',np.mean(python_time),'s'
print "Minimum bei: ",px,',',py,',',pz
print '\n'

print '----Berechnung ohne Schleife komplett mit Numpy-------------------------'
print 'Durchschnittliche Dauer: ',np.mean(numpy_time),'s'
print "Minimum bei: ",npx,',',npy,',',npz
print '\n'

print '----Berechnung mittels Auslagerung in C++-Funktionen--------------------'
print 'Durchschnittliche Dauer: ',np.mean(ctypes_time),'s'
print "Minimum bei: ",cpx,',',cpy,',',cpz
print '\n'



# Plotten
fig = plt.figure(figsize=(10, 7))
ax = fig.gca(projection='3d')
surf = ax.plot_wireframe(X, Y, Z, rstride=10, cstride=10, linewidth=0.5)
min_point1 = ax.scatter(px,py,pz,c=u'r', label="Python")
min_point2 = ax.scatter(npx,npy,npz,c=u'b', label="Numpy")
min_point3 = ax.scatter(cpx,cpy,cpz,c=u'g', label="Ctypes")
ax.set_title("Ackley-Funktion")
ax.set_xlabel("x")
ax.set_ylabel("y")
ax.set_zlabel("z")
plt.show()